import { ImplementableApi } from './implementableApi';

import { DiscordParser } from './discordParser';
import { LogWriter } from './logWriter';
import { PeerTubeRequester } from './peertubeRequester';
import EventEmitter from 'events';

namespace Router {
    export type EventsType = {
        name: string;
        type: 'emit' | 'received';
    };
    export type InternalConfig = {
        events: EventsType[];
        apis: {
            apiName: string;
        }[];
    };

    // export type GlobalConfig = {
    //     router: InternalConfig;
    // };
}

class Router extends EventEmitter {
    api_array: { [key: string]: ImplementableApi } = {};
    routes: Router.InternalConfig;

    constructor(readonly config: Router.EventsType[]) {
        super();
        this.routes = { events: config, apis: [] };
    }

    public injectDependency(api: ImplementableApi): void {
        if (api.name in this.api_array)
            throw `The api name '${api.name}' is already take`;
        this.routes.apis.push({ apiName: api.name });
        this.api_array[api.name] = api;

        this.setEvents(api);
    }

    private setEvents(api: ImplementableApi) {
        for (const eventName in this.routes.events.map(
            (ev) => ev.type === 'received'
        ))
            api.on(eventName, () => this.emit(eventName));
    }

    public receivedMessage(message: ImplementableApi.Message) {
        for (const apiName in this.routes.apis)
            this.api_array[apiName].receivedMessage(message);
    }
}

export { Router };
